#!/usr/bin/env python
# -*- coding: utf-8 -*-
# setup from TinERP
#   taken from straw http://www.nongnu.org/straw/index.html
#   taken from gnomolicious http://www.nongnu.org/gnomolicious/
#   adapted by Nicolas Évrard <nicoe@altern.org>

import imp
import sys
import os
import glob

from stat import ST_MODE

from distutils.file_util import copy_file
from mydistutils import setup

opj = os.path.join

name = 'tinyerp-client'
version = '2.1.2'

required_modules = [('gtk', 'gtk python bindings'),
                    ('gtk.glade', 'glade python bindings')]

def check_modules():
    ok = True
    for modname, desc in required_modules:
        try:
            exec('import %s' % modname)
        except ImportError:
            ok = False
            print 'Error: python module %s (%s) is required' % (modname, desc)

    if not ok:
        sys.exit(1)

def data_files():
    '''Build list of data files to be installed'''
    files = [(opj('share','man','man1',''),['man/tinyerp-client.1']),
             (opj('share','doc', 'tinyerp-client-%s' % version), 
              [f for f in glob.glob('doc/*') if os.path.isfile(f)]),
             (opj('share', 'pixmaps', 'terp-client'),
              glob.glob('pixmaps/*.png') + glob.glob('bin/*.png')),
             (opj('share', 'terp-client'),
              ['bin/terp.glade'])]
    return files

included_plugins = ['auction_lots_html', 'slideshow', 'workflow_print']

def find_plugins():
    for plugin in included_plugins:
        path=opj('bin', 'plugins', plugin)
        for dirpath, dirnames, filenames in os.walk(path):
            if '__init__.py' in filenames:
                modname = dirpath.replace(os.path.sep, '.')
                yield modname.replace('bin', 'terp-client', 1)

def translations():
    trans = []
    dest = 'share/locale/%s/LC_MESSAGES/%s.mo'
    for po in glob.glob('bin/po/*.po'):
        lang = os.path.splitext(os.path.basename(po))[0]
        trans.append((dest % (lang, name), po))
    return trans

long_desc = '''\
Tiny ERP is a complete ERP and CRM. The main features are accounting (analytic
and financial), stock management, sales and purchases management, tasks
automation, marketing campaigns, help desk, POS, etc. Technical features include
a distributed server, flexible workflows, an object database, a dynamic GUI,
customizable reports, and SOAP and XML-RPC interfaces.
'''

classifiers = """\
Development Status :: 5 - Production/Stable
License :: OSI Approved :: GNU General Public License (GPL)
Programming Language :: Python
"""

check_modules()

# todo: use 
command = sys.argv[1]

setup(name             = name,
      version          = version,
      description      = "Tiny's ERP Client",
      long_description = long_desc,
      url              = 'http://tinyerp.com',
      author           = 'Tiny.be',
      author_email     = 'info@tiny.be',
      classifiers      = filter(None, classifiers.splitlines()),
      license          = 'GPL',
      data_files       = data_files(),
      translations     = translations(),
      pot_file         = 'bin/po/terp-msg.pot',
      scripts          = ['bin/tinyerp-client'],
      packages         = ['terp-client', 'terp-client.common', 
                          'terp-client.modules', 'terp-client.modules.action',
                          'terp-client.modules.gui',
                          'terp-client.modules.gui.window',
                          'terp-client.modules.gui.window.view_sel',
                          'terp-client.modules.gui.window.view_tree',
                          'terp-client.modules.spool',
                          'terp-client.printer', 'terp-client.tools',
                          'terp-client.widget',
                          'terp-client.widget.form',
                          'terp-client.widget.search',
                          'terp-client.widget.tree'] + list(find_plugins()),
      package_dir      = {'terp-client': 'bin'},
      )

# this is now done by 'install_script'
#if command == 'install':
#    ver = '%s.%s' % sys.version_info[:2]
#    exec_path = '/usr/lib/python%s/site-packages/terp-client/tinyerp-client.py' % ver
#    copy_file(os.path.join('bin', 'tinyerp-client'), exec_path)
#    copy_file(exec_path ,'/usr/bin/tinyerp-client', link = 'sym')
#    mode = ((os.stat(exec_path)[ST_MODE]) | 0555) & 07777
#    os.chmod(exec_path, mode)

# vim:expandtab:tw=80
